<?php

declare(strict_types=1);

namespace Dedi\SyliusImageOptimizerPlugin\Resolver;

use Exception;
use Liip\ImagineBundle\Imagine\Cache\CacheManager;
use Liip\ImagineBundle\Imagine\Data\DataManager;
use Liip\ImagineBundle\Imagine\Filter\FilterManager;
use Psr\Log\LoggerInterface;

class ImageResolverOnTheFly implements ImageResolverInterface
{
    private readonly CacheManager $cacheManager;
    private readonly DataManager $dataManager;
    private readonly FilterManager $filterManager;
    private readonly LoggerInterface $logger;

    public function __construct(
        LoggerInterface $logger,
        CacheManager $cacheManager,
        DataManager $dataManager,
        FilterManager $filterManager
    ) {
        $this->cacheManager = $cacheManager;
        $this->dataManager = $dataManager;
        $this->filterManager = $filterManager;
        $this->logger = $logger;
    }

    public function resolve(string $filterName, string $imageName, array $options): string
    {
        $dirname = pathinfo($imageName, \PATHINFO_DIRNAME);
        $filename = pathinfo($imageName, \PATHINFO_FILENAME);
        $ext = pathinfo($imageName, \PATHINFO_EXTENSION);

        $filtersOption = [];
        if (array_key_exists('filters', $options)) {
            foreach ($options['filters'] as $filter) {
                $filtersOption[$filter['name']] = $filter['options'];
                $filename = sprintf('%s_%s%s', $filename, $filter['name'], $this->getOptionValue($filter['options']));
            }
        } else {
            $filename = sprintf('%s_%d_%d', $filename, $options['width'], $options['height']);
        }

        $filterConfigs = $this->filterManager->getFilterConfiguration();

        try {
            $filterConfig = $filterConfigs->get($filterName);
            if (\array_key_exists('format', $filterConfig)) {
                $ext = $filterConfig['format'];
            }
            foreach ($filterConfig['filters'] as $key => $filter) {
                $filtersOption[$key] = array_merge($filter, [
                    'size' => [$options['width'], $options['height']],
                ]);
            }
        } catch (Exception $e) {
            $this->logger->error($e->getMessage());
        }

        $filename .= '.' . $ext;
        $cacheName = $dirname !== '.' ? $dirname . \DIRECTORY_SEPARATOR . $filename : $filename;

        try {
            if (!$this->cacheManager->isStored($cacheName, $filterName)) {
                $binary = $this->dataManager->find($filterName, $imageName);

                $filteredBinary = $this->filterManager->applyFilter($binary, $filterName, [
                    'filters' => $filtersOption,
                ]);

                $this->cacheManager->store($filteredBinary, $cacheName, $filterName);
            }

            return $this->cacheManager->resolve($cacheName, $filterName);
        } catch (Exception $e) {
            $this->logger->error($e->getMessage());

            return $imageName;
        }
    }

    /**
     * Concate values of array recursively separate with underscore
     */
    private function getOptionValue(array $options): string
    {
        $valueName = '';

        foreach ($options as $value) {
            if (is_array($value)) {
                $valueName .= $this->getOptionValue($value);
            } else {
                $valueName .= '_' . (is_bool($value) ? (int) (string) $value : $value);
            }
        }

        return $valueName;
    }
}
